import datetime

from kpc_nifi_utils.shared.timezone import TimezoneUtils as tzUtils


class S3Parser:

    @staticmethod
    def compose_path(prefix: str = None, filename: str = None, date_sep: bool = True, file_format: str = 'json',
                     partition: int = 0, datetime_group: datetime = None, timezone: str = 'Asia/Bangkok'):

        if not prefix or not filename:
            raise ValueError('Prefix and filename should be specified.')

        now = tzUtils.utcnow_localize()

        if timezone is not None:
            now = tzUtils.as_timezone(now, timezone)

        if partition > 0:
            if datetime_group is None:
                raise ValueError('datetime_group must be specified if partition is set.')

            now = datetime_group

        if date_sep:
            res = '{}/{}'.format(prefix, now.strftime("%Y/%m/%d/%H%M%S"))
        else:
            res = '{}'.format(prefix)

        if partition > 0:
            return '{}/{}-{}.{}.{}'.format(res, filename, now.isoformat(), partition, file_format)

        return '{}/{}-{}.{}'.format(res, filename, now.isoformat(), file_format)



import kpc_nifi_utils.shared.constants as constants
from kpc_nifi_utils.args.aws import AwsArgs
from kpc_nifi_utils.args.mongodb import MongoArgs
from kpc_nifi_utils.args.mssql import MSSqlArgs
from kpc_nifi_utils.args.postgresql import PostgreSqlArgs
from kpc_nifi_utils.args.s3 import S3Args
from kpc_nifi_utils.common.args_parser import Args
from kpc_nifi_utils.common.json_encoder import EnhancedJSONEncoder
from kpc_nifi_utils.common.singleton import singleton
from kpc_nifi_utils.connector.mongo_connector import MongoConnector
from kpc_nifi_utils.connector.mssql_connector import MSSqlConnector
from kpc_nifi_utils.connector.postgresql_connector import PostgreSqlConnector
from kpc_nifi_utils.nifi.execute_stream_command import Command
from kpc_nifi_utils.nifi.execute_stream_command import Receiver
from kpc_nifi_utils.nifi.execute_stream_command import Sender
from kpc_nifi_utils.parser.s3 import S3Parser
from kpc_nifi_utils.shared.datetime import DatetimeUtils
from kpc_nifi_utils.shared.timezone import TimezoneUtils



import json

import pg8000
from pg8000 import Cursor

import kpc_nifi_utils.shared.constants as constants
from kpc_nifi_utils.args.postgresql import PostgreSqlArgs
from kpc_nifi_utils.common.json_encoder import EnhancedJSONEncoder


class PostgreSqlConnector:
    def __init__(self, connector_args: PostgreSqlArgs = None, map_setting: dict = None):
        self._args = connector_args
        self._map_setting = map_setting

        self._username = None
        self._password = None
        self._host = None
        self._port = None
        self._db_name = None

        if self._args is not None:
            self._username = self._args.get_username()
            self._password = self._args.get_password()
            self._host = self._args.get_host()
            self._port = self._args.get_port()
            self._db_name = self._args.get_dbname()

        if self._map_setting is not None:
            self._username = self._username if self._username else self._map_setting.get(constants.POSTGRESQL_USERNAME)
            self._password = self._password if self._password else self._map_setting.get(constants.POSTGRESQL_PASSWORD)
            self._host = self._host if self._host else self._map_setting.get(constants.POSTGRESQL_HOST)
            self._port = self._port if self._port else self._map_setting.get(constants.POSTGRESQL_PORT)
            self._db_name = self._db_name if self._db_name else self._map_setting.get(constants.POSTGRESQL_DBNAME)

    def connect(self, user=None, host=None, unix_sock=None, port=None, database=None,
                password=None, ssl=None, timeout=None, application_name=None,
                max_prepared_statements=1000, tcp_keepalive=True):

        if user is None and host is None and port is None and self._args is None and self._map_setting is None:
            raise ValueError("Can not get db because there is no setting initialize or pass")

        user = user if user else self._username
        password = password if password else self._password
        host = host if host else self._host
        port = port if port else self._port
        database = database if database else self._db_name

        return pg8000.connect(user=user, host=host, unix_sock=unix_sock, port=port, database=database,
                              password=password, ssl=ssl, timeout=timeout, application_name=application_name,
                              max_prepared_statements=max_prepared_statements, tcp_keepalive=tcp_keepalive)

    @staticmethod
    def fetchall_asdict(cursor: Cursor):
        cols = [a[0].decode("utf-8") for a in cursor.description]
        result = []
        for row in cursor.fetchall():
            result.append({a: b for a, b in zip(cols, row)})

        return result

    @staticmethod
    def fetchone_asdict(cursor: Cursor):
        cols = [a[0].decode("utf-8") for a in cursor.description]
        result = []
        res = cursor.fetchone()
        result.append({a: b for a, b in zip(cols, list(res))})

        return result[0]

    @staticmethod
    def fetchmany_asdict(cursor: Cursor, num: int):
        cols = [a[0].decode("utf-8") for a in cursor.description]
        result = []
        for row in cursor.fetchmany(num):
            result.append({a: b for a, b in zip(cols, row)})

        return result

    @staticmethod
    def parse_as_json(data):
        data_parse = data
        if isinstance(data, dict):
            data_parse = [data]

        return json.dumps(data_parse, cls=EnhancedJSONEncoder)



import datetime

import pytz


class TimezoneUtils:

    @staticmethod
    def utcnow_localize():
        return pytz.utc.localize(datetime.datetime.utcnow())

    @staticmethod
    def as_timezone(dt: datetime, tz: str = 'Asia/Bangkok'):
        if dt.tzinfo is None:
            dt = pytz.utc.localize(dt)

        zone = pytz.timezone(tz)
        return dt.astimezone(zone)



import datetime
import decimal
import json
from uuid import UUID

from bson import ObjectId


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        elif isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, datetime.time):
            return obj.isoformat()
        elif isinstance(obj, datetime.timedelta):
            return obj.seconds
        elif isinstance(obj, decimal.Decimal):
            return float(str(obj))
        elif isinstance(obj, ObjectId):
            return str(obj)
        elif isinstance(obj, bytes):
            return obj.decode(encoding="utf8", errors='ignore')
        elif isinstance(obj, UUID):
            return obj.hex
        else:
            return super().default(obj)



from kpc_nifi_utils.args.aws import AwsArgs
from kpc_nifi_utils.shared import constants


class S3Args(AwsArgs):
    def __init__(self, *args):
        super().__init__(*args)

    def get_bucket(self):
        return self.get(constants.AWS_S3_BUCKET)



from kpc_nifi_utils.common.args_parser import Args
from kpc_nifi_utils.shared import constants


class PostgreSqlArgs(Args):
    def __init__(self, *args):
        super().__init__(*args)

    def get_username(self):
        return self.get(constants.POSTGRESQL_USERNAME)

    def get_password(self):
        return self.get(constants.POSTGRESQL_PASSWORD)

    def get_host(self):
        return self.get(constants.POSTGRESQL_HOST)

    def get_port(self):
        return int(self.get(constants.POSTGRESQL_PORT)) if self.get(constants.POSTGRESQL_PORT) else None

    def get_dbname(self):
        return self.get(constants.POSTGRESQL_DBNAME)
